@@ -3,7 +3,7 @@ package YAHC;
 use strict;
 use warnings;
 
-our $VERSION = '0.025';
+our $VERSION = '0.026';
 
 use EV;
 use Time::HiRes;
@@ -65,6 +65,7 @@ our @EXPORT_OK = qw/
     yahc_conn_request
     yahc_conn_response
     yahc_conn_attempts_left
+    yahc_conn_socket_cache_id
 /;
 
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
@@ -91,6 +92,7 @@ sub new {
         last_connection_id  => $$ * 1000,
         debug               => delete $args->{debug} || $ENV{YAHC_DEBUG} || 0,
         keep_timeline       => delete $args->{keep_timeline} || $ENV{YAHC_TIMELINE} || 0,
+        socket_cache        => delete $args->{socket_cache},
         pool_args           => $args,
     }, $class;
 
@@ -98,6 +100,13 @@ sub new {
     # let's see how it plays out in practise.
     weaken($self->{storage});
     weaken($self->{$_} = $storage{$_} = {}) for qw/watchers callbacks connections/;
+
+    if (delete $args->{account_for_signals}) {
+        _log_message('YAHC: enable account_for_signals logic') if $self->{debug};
+        my $sigcheck = $self->{watchers}{_sigcheck} = $self->{loop}->check(sub {});
+        $sigcheck->keepalive(0);
+    }
+
     return $self, \%storage;
 }
 
@@ -122,8 +131,6 @@ sub request {
     }
 
     my $scheme = $request->{scheme} ||= 'http';
-    die "YAHC: only support scheme http and https\n" unless $scheme eq 'http' || $scheme eq 'https';
-
     my $debug = delete $request->{debug} || $self->{debug};
     my $keep_timeline = delete $request->{keep_timeline} || $self->{keep_timeline};
 
@@ -143,8 +150,8 @@ sub request {
     my %callbacks;
     foreach (@{ CALLBACKS() }) {
         next unless exists $request->{$_};
-        $callbacks{$_} = delete $request->{$_};
-        $conn->{"has_$_"} = 1;
+        my $cb = $callbacks{$_} = delete $request->{$_};
+        $conn->{"has_$_"} = !!$cb;
     }
 
     $self->{watchers}{$conn_id} = {};
@@ -182,6 +189,19 @@ sub run_tick    { shift->_run(EV::RUN_NOWAIT)   }
 sub is_running  { !!shift->{loop}->depth        }
 sub loop        { shift->{loop}                 }
 
+sub break {
+    my ($self, $reason) = @_;
+    return unless $self->is_running;
+    _log_message('YAHC: pid %d breaking event loop because %s', $$, ($reason || 'no reason')) if $self->{debug};
+    $self->{loop}->break(EV::BREAK_ONE)
+}
+
+sub socket_cache {
+    my $self = shift;
+    $self->{socket_cache} = $_[0] if @_;
+    return $self->{socket_cache};
+}
+
 ################################################################################
 # Routines to manipulate connections (also user facing)
 ################################################################################
@@ -254,6 +274,7 @@ sub _run {
         _log_message('YAHC: reinitializing event loop after forking') if $self->{debug};
         $self->{pid} = $$;
         $self->{loop}->loop_fork;
+        $self->{socket_cache} = {} if defined $self->{socket_cache};
     }
 
     if (defined $until_state) {
@@ -287,12 +308,6 @@ sub _run {
     }
 }
 
-sub _break {
-    my ($self, $reason) = @_;
-    _log_message('YAHC: pid %d breaking event loop because %s', $$, ($reason || 'no reason')) if $self->{debug};
-    $self->{loop}->break(EV::BREAK_ONE)
-}
-
 sub _check_stop_condition {
     my ($self, $conn) = @_;
     my $stop_condition = $self->{stop_condition};
@@ -303,7 +318,7 @@ sub _check_stop_condition {
     my $expected_state = $stop_condition->{expected_state};
 
     if ($awaiting_connections == 0) {
-        $self->_break(sprintf("until state '%s' is reached", _strstate($expected_state)));
+        $self->break(sprintf("until state '%s' is reached", _strstate($expected_state)));
         return 1;
     }
 
@@ -318,6 +333,7 @@ sub _check_stop_condition {
 sub _set_init_state {
     my ($self, $conn_id) = @_;
 
+    my $socket_cache = $self->{socket_cache};
     my $conn = $self->{connections}{$conn_id}  or die "YAHC: unknown connection id $conn_id\n";
     my $watchers = $self->{watchers}{$conn_id} or die "YAHC: no watchers for connection id $conn_id\n";
 
@@ -328,9 +344,7 @@ sub _set_init_state {
 
     my $continue = 1;
     while ($continue) {
-        delete $watchers->{io}; # implicit stop
-        my $fh = delete $watchers->{_fh};
-        $fh && close($fh), undef $fh;
+        _close_or_cache_socket($self, $conn, 1); # force connection close if any (likely not)
 
         if ($conn->{attempt} > $conn->{retries}) {
             _set_user_action_state($self, $conn_id, YAHC::Error::CONNECT_ERROR(), "retries limit reached");
@@ -352,8 +366,17 @@ sub _set_init_state {
             my ($host, $ip, $port, $scheme) = _get_next_target($conn);
             _register_in_timeline($conn, "Target $scheme://$host:$port ($ip:$port) chosen for attempt #$attempt") if exists $conn->{debug_or_timeline};
 
-            my $sock = _build_socket_and_connect($ip, $port, $conn->{request});
-            _set_wait_synack_state($self, $conn_id, $sock);
+            my $socket_cache_id; $socket_cache_id = yahc_conn_socket_cache_id($conn) if defined $socket_cache;
+            if (defined $socket_cache_id && exists $socket_cache->{$socket_cache_id}) {
+                _register_in_timeline($conn, "reuse socket '$socket_cache_id'") if $conn->{debug_or_timeline};
+                my $sock = $watchers->{_fh} = delete $socket_cache->{$socket_cache_id};
+                $watchers->{io} = $self->{loop}->io($sock, EV::WRITE, sub {});
+                _set_write_state($self, $conn_id);
+            } else {
+                my $sock = _build_socket_and_connect($ip, $port, $conn->{request});
+                _set_wait_synack_state($self, $conn_id, $sock);
+            }
+
             $continue = 0;
             1;
         } or do {
@@ -445,7 +468,7 @@ sub _set_ssl_handshake_state {
 
     if (!IO::Socket::SSL->start_SSL($fh, %options, SSL_startHandshake => 0)) {
         _register_error($conn, YAHC::Error::SSL_ERROR(), "Failed to start SSL session: $IO::Socket::SSL::SSL_ERROR");
-        return _set_completed_state($self, $conn_id);
+        return _set_completed_state($self, $conn_id, 1); # XXX should be _set_init_state() ???
     }
 
     my $handshake_cb = _get_safe_wrapper($self, $conn, sub {
@@ -484,6 +507,9 @@ sub _set_write_state {
     my $buf = _build_http_message($conn);
     my $length = length($buf);
 
+    warn "YAHC: HTTP message has UTF8 flag set! This will result in poor performance, see docs for details!"
+        if utf8::is_utf8($buf);
+
     _register_in_timeline($conn, "sending request of $length bytes") if exists $conn->{debug_or_timeline};
 
     my $write_cb = _get_safe_wrapper($self, $conn, sub {
@@ -647,11 +673,12 @@ sub _set_user_action_state {
     _register_in_timeline($conn, "new state %s", _strstate($conn->{state})) if exists $conn->{debug_or_timeline};
     _register_error($conn, $error, $strerror) if $error;
 
+    _close_or_cache_socket($self, $conn, $error != YAHC::Error::NO_ERROR);
     return _set_completed_state($self, $conn_id) unless exists $conn->{has_callback};
-    my $cb = $self->{callbacks}{$conn_id}{callback};
 
     eval {
         _register_in_timeline($conn, "call callback%s", $error ? " error=$error, strerror='$strerror'" : '') if exists $conn->{debug_or_timeline};
+        my $cb = $self->{callbacks}{$conn_id}{callback};
         $cb->($conn, $error, $strerror);
         1;
     } or do {
@@ -677,12 +704,11 @@ sub _set_user_action_state {
 }
 
 sub _set_completed_state {
-    my ($self, $conn_id) = @_;
+    my ($self, $conn_id, $force_socket_close) = @_;
 
     # this's a terminal state,
     # so setting this state should *NEVER* fail
     delete $self->{callbacks}{$conn_id};
-    my $watchers = delete $self->{watchers}{$conn_id};
     my $conn = $self->{connections}{$conn_id}
       or warn "YAHC: try to _set_completed_state() for unknown connection $conn_id",
         return;
@@ -690,9 +716,8 @@ sub _set_completed_state {
     $conn->{state} = YAHC::State::COMPLETED();
     _register_in_timeline($conn, "new state %s", _strstate($conn->{state})) if exists $conn->{debug_or_timeline};
 
-    my $fh = $watchers->{_fh};
-    $fh && close($fh);
-    undef $watchers; # implicit stop
+    _close_or_cache_socket($self, $conn, $force_socket_close);
+    delete $self->{watchers}{$conn_id}; # implicit stop of all watchers
 
     _check_stop_condition($self, $conn) if exists $self->{stop_condition};
 }
@@ -730,6 +755,51 @@ sub _get_next_target {
     return @{ $conn->{selected_target} = [ $host, $ip, $port, $scheme ] };
 }
 
+# this and following functions are used in terminal state
+# so they should *NEVER* fail
+sub _close_or_cache_socket {
+    my ($self, $conn, $force_close) = @_;
+    my $watchers = $self->{watchers}{$conn->{id}} or return;
+    my $fh = delete $watchers->{_fh} or return;
+    delete $watchers->{io}; # implicit stop
+
+    my $socket_cache = $self->{socket_cache};
+    my $socket_cache_id; $socket_cache_id = yahc_conn_socket_cache_id($conn) if defined $socket_cache;
+
+    # Stolen from Hijk. Thanks guys!!!
+    # We always close connections for 1.0 because some servers LIE
+    # and say that they're 1.0 but don't close the connection on
+    # us! An example of this. Test::HTTP::Server (used by the
+    # ShardedKV::Storage::Rest tests) is an example of such a
+    # server. In either case we can't cache a connection for a 1.0
+    # server anyway, so BEGONE!
+
+    if (   $force_close
+        || !defined $socket_cache_id
+        || (($conn->{request}{proto} || '') eq 'HTTP/1.0')
+        || (($conn->{response}{head}{Connection} || '') eq 'close'))
+    {
+        _register_in_timeline($conn, "drop socket %s", $socket_cache_id || '<noyahc_conn_socket_cache_id>') if $conn->{debug_or_timeline};
+        close($fh);
+        return;
+    }
+
+    if (exists $socket_cache->{$socket_cache_id}) {
+        close(delete $socket_cache->{$socket_cache_id});
+    }
+
+    $socket_cache->{$socket_cache_id} = $fh;
+    _register_in_timeline($conn, "save socket '$socket_cache_id' for later use") if $conn->{debug_or_timeline};
+}
+
+sub yahc_conn_socket_cache_id {
+    my $conn = shift;
+    my ($host, $ip, $port, $scheme) = @{ $conn->{selected_target} || [] };
+    return unless $host && $port && $scheme;
+    # Use $; so we can use the $socket_cache->{$$, $host, $port} idiom to access the cache.
+    return join($;, $$, $host, $port, $scheme);
+}
+
 ################################################################################
 # Timers
 ################################################################################
@@ -859,7 +929,7 @@ sub _get_safe_wrapper {
         my $error = $@ || 'zombie error';
         _register_error($conn, YAHC::Error::INTERNAL_ERROR(), "Exception callback: $error");
         warn "YAHC: exception in callback: $error";
-        _set_completed_state($self, $conn->{id});
+        _set_completed_state($self, $conn->{id}, 1);
     }};
 }
 
@@ -942,7 +1012,7 @@ roughly descending order of feature completeness) L<LWP::UserAgent>,
 L<WWW::Curl>, L<HTTP::Tiny>, L<HTTP::Lite> or L<Furl>. This library is
 basically one step above manually talking HTTP over sockets.
 
-YAHC supports SSL.
+YAHC supports SSL and socket reuse (later is in experimental mode).
 
 =head1 STATE MACHINE
 
@@ -981,39 +1051,102 @@ Each YAHC connection goes through following list of states in its lifetime:
                   +-----------------+
 
 
-There are three main paths:
+There are three paths of workflow:
 
 =over 4
 
 =item 1) Normal execution (central line).
 
-    In normal situation a connection after being initialized goes through state:
-    - RESOLVE DNS
-    - WAIT SYNACK - wait finishing of handshake
-    - CONNECTED
-    - WRITTING - sending request body
-    - READING - awaiting and reading response
-    - USER ACTION - see below
-    - COMPLETED - all done, this is terminal state
+In normal situation a connection after being initialized goes through state:
 
-    * SSL connection has extra state SSL_HANDSHAKE after CONNECTED state.
-    * State 'RESOLVE DNS' is not implemented yet.
+- RESOLVE DNS
+
+- WAIT SYNACK - wait finishing of handshake
+
+- CONNECTED
+
+- WRITTING - sending request body
+
+- READING - awaiting and reading response
+
+- USER ACTION - see below
+
+- COMPLETED - all done, this is terminal state
+
+SSL connection has extra state SSL_HANDSHAKE after CONNECTED state. State
+'RESOLVE DNS' is not implemented yet.
 
 =item 2) Retry path (right line).
 
-    In case of IO error during normal execution YAHC retries connection
-    C<retries> times. In practise this means that connection goes back to
-    INITIALIZED state.
+In case of IO error during normal execution YAHC retries connection
+C<retries> times. In practise this means that connection goes back to
+INITIALIZED state.
 
-    * It's possible for a connection to go directly to COMPLETED state in case of internal error.
+It's possible for a connection to go directly to COMPLETED state in case
+of internal error.
 
 =item 3) Failure path (left line).
 
-    If all retry attempts did not succeeded a connection goes to state 'USER ACTION' (see below).
+If all retry attempts did not succeeded a connection goes to state 'USER
+ACTION' (see below).
 
 =back
 
-=head1 State 'USER ACTION'
+=head2 State 'USER ACTION'
+
+'USER ACTION' state is called right before connection if going to enter
+'COMPLETED' state (with either failed or successful results) and is meant
+to give a change to user to interupt the workflow.
+
+'USER ACTION' state is entered in these circumstances:
+
+=over 4
+
+=item * HTTP response received. Note that non-200 responses are NOT treated as error. 
+
+=item * unsupported HTTP response is received (such as response without Content-Length header
+
+=item * retries limit reached
+
+=back
+
+When a connection enters this state C<callback> CodeRef is called:
+
+    $yahc->request({
+        ...
+        callback => sub {
+            my (
+                $conn,          # connection 'object'
+                $error,         # one of YAHC::Error::* constants
+                $strerror       # string representation of error
+            ) = @_;
+
+            # Note that fields in $conn->{response} are not set 
+            # if $error != # YAHC::Error::NO_ERROR()
+
+            # HTTP response is stored in $conn->{response}.
+            # It can be also accessed via yahc_conn_response().
+            my $response = $conn->{response};
+            my $status = $response->{status};
+            my $body = $response->{body};
+        }
+    });
+
+If there was no IO error C<yahc_conn_response> return value is a C<HashRef>
+representing a response. It contains the following key-value pairs.
+
+    proto         => :Str
+    status        => :StatusCode
+    body          => :Str
+    head          => :HashRef
+
+In case of error or non-200 HTTP response C<yahc_retry_conn> or
+C<yahc_reinit_conn> may be called to give the request more chances to complete
+successfully (for example by following redirects or providing new target
+hosts).
+
+Note that C<callback> should NOT throw exception. If so the connection will be
+imidiately closed.
 
 =head1 METHODS
 
@@ -1023,7 +1156,7 @@ This method creates YAHC object and accompanying storage object:
 
     my ($yahc, $yahc_storage) = YAHC->new();
 
-This's a radical way of solving all possible memleak because of cyclic
+This is a radical way of solving all possible memleak because of cyclic
 references in callbacks. Since all references of callbacks are kept in
 $yahc_storage object it's fine to use YAHC object inside request callback:
 
@@ -1033,11 +1166,66 @@ $yahc_storage object it's fine to use YAHC object inside request callback:
         },
     });
 
-However, use has to garantee that both $yahc and $yahc_storage objects are kept
-in the same namespace. So, they will be destroyed at the same time.
+However, user has to guarantee that both $yahc and $yahc_storage objects are
+kept in the same namespace. So, they will be destroyed at the same time.
 
-This method can be passed with all parameters supported by C<request>. They
-will be inhereted by all requests.
+C<new> can be passed with all parameters supported by C<request>. They
+will be inherited by all requests.
+
+Additionally, C<new> supports two parameters: C<socket_cache> and
+C<account_for_signals>.
+
+=head3 socket_cache
+
+C<socket_cache> option controls socket reuse logic. By default socket cache is
+disabled. If user wants YAHC reuse sockets he should set C<socket_cache> to a
+HashRef.
+
+    my ($yahc, $yahc_storage) = YAHC->new({ socket_cache => {} });
+
+In this case YAHC maintains unused sockets keyed on C<join($;, $$, $host,
+$port, $scheme)>. We use $; so we can use the $socket_cache->{$$, $host, $port,
+$scheme} idiom to access the cache.
+
+It's up to user to control the cache. It's also up to user to set necessary
+request headers for keep-alive. YAHC does not cache socket in cases of a error,
+HTTP/1.0 and when server explicitly instruct to close connection (i.e header
+'Connection' = 'close').
+
+=head3 account_for_signals
+
+Another parameter C<account_for_signals> requires special attention! Here is
+why:
+
+=over 4
+
+exerpt from EV documentation http://search.cpan.org/~mlehmann/EV-4.22/EV.pm#PERL_SIGNALS
+
+While Perl signal handling (%SIG) is not affected by EV, the behaviour with EV
+is as the same as any other C library: Perl-signals will only be handled when
+Perl runs, which means your signal handler might be invoked only the next time
+an event callback is invoked.
+
+=back
+
+In practise this means that none of set %SIG handlers will be called until EV
+calls one of perl callbacks. Which, in some cases, may take long time. By
+setting C<account_for_signals> YAHC adds C<EV::check> watcher with empty
+callback effectively making EV calling the callback on every iteration. The
+trickery comes at some performance cost. This is what EV documentation says
+about it:
+
+=over 4
+
+... you can also force a watcher to be called on every event loop iteration by
+installing a EV::check watcher. This ensures that perl gets into control for a
+short time to handle any pending signals, and also ensures (slightly) slower
+overall operation.
+
+=back
+
+So, if your code or the codes surrounding your code use %SIG handlers it's
+wise to set C<account_for_signals>.
 
 =head2 request
 
@@ -1079,6 +1267,8 @@ the following parameters:
         query_string => "color=red"
     });
 
+=head3 request building
+
 YAHC doesn't escape any values for you, it just passes them through
 as-is. You can easily produce invalid requests if e.g. any of these
 strings contain a newline, or aren't otherwise properly escaped.
@@ -1102,6 +1292,28 @@ Will produce these request headers:
     Content-Type: application/json
     X-Requested-With: YAHC
 
+=head3 host
+
+C<host> parameter can accept one of following values:
+
+=over 4
+
+    1) string - represents target host. String may have following formats:
+    hostname:port, ip:port.
+
+    2) ArrayRef of strings - YAHC will cycle through items selecting new host
+    for each attempt.
+
+    3) CodeRef. The subroutine is invoked for each attempt and should at least
+    return a string (hostname or IP address). It can also return array
+    containing: ($host, $ip, $port, $scheme). This option effectively give a
+    user control over host selection for retries. The CodeRef is passed with
+    connection "object" which can be fed to yahc_conn_* family of functions.
+
+=back
+
+=head3 timeouts
+
 The value of C<connect_timeout>, C<request_timeout> and C<drain_timeout> is in
 floating point seconds, and is used as the time limit for connecting to the
 host (reaching CONNECTED state), full request time (reaching COMPLETED state)
@@ -1110,18 +1322,18 @@ default value for all is C<undef>, meaning no timeout limit. If you don't
 supply these timeouts and the host really is unreachable or slow, we'll reach
 the TCP timeout limit before returning some other error to you.
 
+=head3 callbacks
+
 The value of C<init_callback>, C<wait_synack_callback>, C<connected_callback>,
 C<writing_callback>, C<reading_callback> is CodeRef to a subroutine which is
 called upon reaching corresponding state. Any exception thrown in the
-subroutine moves connection to COMPLETED state effectivly terminating any
+subroutine moves connection to COMPLETED state effectively terminating any
 ongoing IO.
 
-The value of C<callback> defines main request callback.
-TODO
+The value of C<callback> defines main request callback which is called when a
+connection enters 'USER ACTION' state (see 'USER ACTION' state above).
 
-We currently don't support servers returning a http body without an accompanying
-C<Content-Length> header; bodies B<MUST> have a C<Content-Length> or we won't pick
-them up.
+Also see L<LIMITATIONS>
 
 =head2 drop
 
@@ -1169,16 +1381,45 @@ Return true if YAHC is running, false otherwise.
 
 Return underlying EV loop object.
 
+=head2 break
+
+Break running EV loop if any.
+
 =head1 EXPORTED FUNCTIONS
 
 =head2 yahc_reinit_conn
 
-TODO
+C<yahc_reinit_conn> reinitialize given connection. The attempt counter is reset
+to 0. The function accepts HashRef as second argument. By passing it one can
+change host, port, scheme, body, head and others parameters. The format and
+meaning of these parameters is same as in C<request> method.
+
+One of use cases of C<yahc_reinit_conn>, for example, is to handle redirects:
+
+    use YAHC qw/yahc_reinit_conn/;
+
+    my ($yahc, $yahc_storage) = YAHC->new();
+    $yahc->request({
+        host => 'domain_which_returns_301.com',
+        callback => sub {
+            my $conn = $_[0];
+            yahc_reinit_conn($conn, { host => 'www.newtarget.com' })
+                if $_[0]->{response}{status} == 301;
+        }
+    });
+
+    $yahc->run;
+
+C<yahc_reinit_conn> is meant to be called inside C<callback> i.e. when
+connection is in 'USER ACTION' state.
 
 =head2 yahc_retry_conn
 
 Retries given connection. C<yahc_retry_conn> should be called only if
-C<yahc_conn_attempts_left> returns positive value. Otherwise, exists silently.
+C<yahc_conn_attempts_left> returns positive value. Otherwise, it exits silently.
+
+C<yahc_conn_attempts_left> is meant to be called inside C<callback> similarly
+to C<yahc_reinit_conn>.
 
 =head2 yahc_conn_id
 
@@ -1191,16 +1432,31 @@ Retrun state of given connection
 =head2 yahc_conn_target
 
 Return selected host and port for current attempt for given connection.
-Format "host:port". Default port values are omited.
+Format "host:port". Default port values are omitted.
 
 =head2 yahc_conn_url
 
 Same as C<yahc_conn_target> but return full URL
 
 =head2 yahc_conn_errors
+
+Return errors appeared in given connection. Note that the function returns all
+errors, not only ones happened during current attempt. Returned value is
+ArrayRef of ArrayRefs. Later one represents a error and contains following
+items:
+
+=over 4
+
+    error number (see YAHC::Error constants)
+    error string
+    ArrayRef of host, ip, port, scheme
+    time when the error happened
+
+=back
+
 =head2 yahc_conn_last_error
 
-TODO
+Return last error appeared in connection. See C<yahc_conn_errors>.
 
 =head2 yahc_conn_timeline
 
@@ -1219,13 +1475,43 @@ Return response of given connection. See C<request>.
 
 L<https://github.com/ikruglov/YAHC>
 
+=head1 NOTES
+
+=head2 UTF8 flag
+
+Note that YAHC has astonishing reduction in performance if any parameters
+participating in building HTTP message has UTF8 flag set. Those fields are
+C<protocol>, C<host>, C<port>, C<method>, C<path>, C<query_string>, C<head>,
+C<body> and maybe others.
+
+Just one example (check scripts/utf8_test.pl for code). Simple HTTP request
+with 10MB of payload:
+
+    elapsed without utf8 flag: 0.039s
+    elapsed with utf8 flag: 0.540s
+
+Because of this YAHC warns if detected UTF8-flagged payload. The user needs
+to make sure that *all* data passed to YAHC is unflagged binary strings.
+
+=head2 LIMITATIONS
+
+=over 4
+
+=item * State 'RESOLVE DNS' is not implemented yet.
+
+=item * YAHC currently don't support servers returning a http body without an
+accompanying C<Content-Length> header; bodies B<MUST> have a C<Content-Length>
+or we won't pick them up.
+
+=back
+
 =head1 AUTHORS
 
 Ivan Kruglov <ivan.kruglov@yahoo.com>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2013 Ivan Kruglov C<< <ivan.kruglov@yahoo.com> >>.
+Copyright (c) 2013-2016 Ivan Kruglov C<< <ivan.kruglov@yahoo.com> >>.
 
 =head1 ACKNOWLEDGMENT
 
